/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU General Public
 * License Version 2, June 1991. See the header in SS_Frame.java
 * and the file License.html for details.
 */

package  ShoeSearch;

import java.awt.*;
import UserInt.GridBagCreator;
import UserInt.HorSeparator;

public class  WebQHelp  extends Dialog
                        implements  ThesaurusClientParent
{
    WebSearchHelpContainer controlWin;
    Frame      parent;
    ThesServe  communication;

    private  TextField  tfPhrase = null;
    private  List    broadTerms = null;
    private  List    narrowTerms = null;
    private  List    relatedTerms = null;
    private  Button  bDoSearch = null;
    private  Button  bGetTerms = null;
    private  Button  bCopy = null;
    private  Button  bQuit = null;

    private  Checkbox   thes[];
    //these are thesauri also:
    private  Checkbox   src[];  //will match thes[i], etc.
    private  CheckboxGroup   sourceCBgrp = null;
    private  TextField  phrSearched[];
    private  TextField  bt[];
    private  TextField  nt[];
    private  TextField  rt[];
    private  TextField  msg;

    static  final  String  SEARCH_LABEL = "Search Phrase: ";
    static  final  String  THESAURI_LABEL = "Which Thesauri to Search:";
    static  final  String  THESNAME_LABEL = "Source";
    static  final  String  PHRASELOOKED_LABEL = "Phrase Searched";
    static  final  String  NUMBT_LABEL = "# BTs";
    static  final  String  NUMNT_LABEL = "# NTs";
    static  final  String  NUMRT_LABEL = "# RTs";
    static  final  String  BROAD_LABEL = "Broad Terms";
    static  final  String  NARROW_LABEL = "Narrow Terms";
    static  final  String  RELATED_LABEL = "Related Terms";

    static  final  String  SEARCH_BUTTON_LABEL = "Do Search";
    static  final  String  TERM_BUTTON_LABEL = "Get Terms";
    static  final  String  COPY_BUTTON_LABEL = "Copy to Web Search";
    static  final  String  QUIT_BUTTON_LABEL = "Quit";

    static  int  RETURN_ASCII = 1;   //tell server to use ascii not html format
    static  int  ROWS_VISIBLE = 10;
    static  final int    NUM_THESAURI = 5;
    /* Important Dependency:
       For 'thesName' and 'responseID", the order is significant;
       the index to the name is the same as the Thesaurus ID in the
       Thesaurus Server.
       (An index of "100" means all the possible thesauri to the server)
     */
    static  String[]  thesName = { "EIStem", "JCTStem", "DTICStem",
                               "UMLS", "nasaStem" };
    private String[] responseID;

    static  int  ALL_THESAURI = 100; // server's code for "search all 5 thes."
               //this is the other dependency related to the thes. server

    /* Methods */

    public WebQHelp (Frame pwin, WebSearchHelpContainer controlWin,
                            String title)
    {
        super (pwin, title, false);
        parent = pwin;
        this.controlWin = controlWin;

        thes = new Checkbox[NUM_THESAURI];
        src = new Checkbox[NUM_THESAURI];
        phrSearched = new TextField[NUM_THESAURI];
        bt = new TextField[NUM_THESAURI];
        nt = new TextField[NUM_THESAURI];
        rt = new TextField[NUM_THESAURI];
        responseID = new String[NUM_THESAURI];
        for (int i = 0; i < NUM_THESAURI; i++)
           responseID[i] = "";   //init to nonexistent value

        createDisplay();

        communication = null;
    }

    protected void finalize()
    {
        communication.disconnect();
        communication = null;
    }

    private void createDisplay()
    {
        GridBagCreator   gbcreat = new GridBagCreator (this);

        Panel  top = createThesSearch();
        HorSeparator  s1 = new HorSeparator();
        Panel  searchRes = createSearchResults();
        HorSeparator  s2 = new HorSeparator();
        Panel  expandRes = createExpandResults();
        Panel  cmdButtons = createCmdButtons();
        msg = new TextField();
        msg.setEditable(false);

        int i = 0;
        gbcreat.addComponent (top, 0, i, GridBagConstraints.REMAINDER, 1,
              100, 0, GridBagConstraints.BOTH, 1, 4);
        i++;
        gbcreat.addComponent (s1, 0, i, GridBagConstraints.REMAINDER, 1,
                        100, 0, GridBagConstraints.HORIZONTAL, 1, 1);
        i++;
        gbcreat.addComponent (searchRes, 0, i, GridBagConstraints.REMAINDER, 1,
              100, 0, GridBagConstraints.BOTH, 1, 2);
        i++;
        gbcreat.addComponent (s2, 0, i, GridBagConstraints.REMAINDER, 1,
                        100, 0, GridBagConstraints.HORIZONTAL, 1, 2);
        i++;
        gbcreat.addComponent (expandRes, 0, i, GridBagConstraints.REMAINDER, 1,
              100, 100, GridBagConstraints.BOTH, 1, 2);
        i++;
        gbcreat.addComponent (cmdButtons, 0, i, GridBagConstraints.REMAINDER, 1,
              100, 0, GridBagConstraints.BOTH, 1, 2);
        i++;
        gbcreat.addComponent (msg, 0, i, GridBagConstraints.REMAINDER, 1,
              100, 0, GridBagConstraints.BOTH, 1, 2);

        pack();
        validate();
    }

    Panel createThesSearch()
    {
        Panel  pan = new Panel();

        Label  search = new Label (SEARCH_LABEL, Label.RIGHT);
        Label  theslabel = new Label (THESAURI_LABEL, Label.CENTER);
        tfPhrase = new TextField("", 30);
        bDoSearch = new Button (SEARCH_BUTTON_LABEL);
        for (int i = 0; i < NUM_THESAURI; i++)
            thes[i] = new Checkbox (thesName[i], null, true);

        GridBagCreator   gbcreat = new GridBagCreator (pan);

        gbcreat.setAnchor (GridBagConstraints.WEST);
        gbcreat.addComponent (search,    0, 0, 1, 1,  0, 0,
                                  GridBagConstraints.NONE, 0, 0);
        gbcreat.addComponent (tfPhrase,  1, 0, 1, 1,  0, 0,
                                  GridBagConstraints.HORIZONTAL, 0, 0);
        gbcreat.setAnchor (GridBagConstraints.CENTER);
        gbcreat.addComponent (bDoSearch, 2, 0, 1, 1,  0, 0,
                                  GridBagConstraints.NONE, 0, 0);
        gbcreat.addComponent (theslabel, 0, 1, GridBagConstraints.REMAINDER, 1,
                                  0, 0,
                                  GridBagConstraints.HORIZONTAL, 0, 0);

        int  row = 1, //set up so first row getting thes[i] comps is row 2
             col = 0;
        int  rowconst = 2;
        for (int i = 0; i < NUM_THESAURI; i++)
        {
            if ( (i % 3) == 0)
                row++;
            col = i % 3;
            gbcreat.addComponent (thes[i], col, row, 1, 1, .33, 0,
                                  GridBagConstraints.BOTH, 0, 0);
        }

        return  pan;
    }

    Panel createSearchResults()
    {
        Panel  pan = new Panel();

        Label  thesnam = new Label (THESNAME_LABEL, Label.LEFT);
        Label  phrLooked = new Label (PHRASELOOKED_LABEL, Label.LEFT);
        Label  bNum = new Label (NUMBT_LABEL, Label.LEFT);
        Label  nNum = new Label (NUMNT_LABEL, Label.LEFT);
        Label  rNum = new Label (NUMRT_LABEL, Label.LEFT);
        sourceCBgrp = new CheckboxGroup();

        GridBagCreator   gbcreat = new GridBagCreator (pan);

        int  row = 1, col;
        for (int i = 0; i < NUM_THESAURI; i++)
        {
            col = 0;
            src[i] = new Checkbox (thesName[i], sourceCBgrp, false);
            gbcreat.addComponent (src[i], col, row, 1, 1, 0, 0,
                                 GridBagConstraints.BOTH, 0, 0);
            col++;
            phrSearched[i] = new TextField ("");
            phrSearched[i].setEditable(false);
            gbcreat.addComponent (phrSearched[i], col, row, 1, 1, 0, 0,
                                 GridBagConstraints.BOTH, 0, 0);
            col++;
            bt[i] = new TextField ("", 6);
            bt[i].setEditable(false);
            gbcreat.addComponent (bt[i], col, row, 1, 1, 0, 0,
                                 GridBagConstraints.BOTH, 0, 0);
            col++;
            nt[i] = new TextField ("", 6);
            nt[i].setEditable(false);
            gbcreat.addComponent (nt[i], col, row, 1, 1, 0, 0,
                            GridBagConstraints.BOTH, 0, 0);
            col++;
            rt[i] = new TextField ("", 6);
            rt[i].setEditable(false);
            gbcreat.addComponent (rt[i], col, row, 1, 1, 0, 0,
                            GridBagConstraints.BOTH, 0, 0);
            row++;
        }

        gbcreat.addComponent (thesnam,        0, 0, 1, 1, .22, 0,
                                 GridBagConstraints.HORIZONTAL, 0, 0);
        gbcreat.addComponent (phrLooked,      1, 0, 1, 1, .30, 0,
                                 GridBagConstraints.HORIZONTAL, 0, 0);
        gbcreat.addComponent (bNum,           2, 0, 1, 1, .16, 0,
                                 GridBagConstraints.HORIZONTAL, 0, 0);
        gbcreat.addComponent (nNum,           3, 0, 1, 1, .16, 0,
                                 GridBagConstraints.HORIZONTAL, 0, 0);
        gbcreat.addComponent (rNum,           4, 0, 1, 1, .16, 0,
                                 GridBagConstraints.HORIZONTAL, 0, 0);

        return  pan;
    }

    private Panel setupList (Label listLabel, List lst)
    {
        Panel  p = new Panel();
        p.setLayout (new BorderLayout());
        p.add (BorderLayout.NORTH, listLabel);
        p.add (BorderLayout.CENTER, lst);
        return   p;
    }

    Panel createExpandResults()
    {
        Panel  pan = new Panel();
        Panel  p1, p2, p3;

        Label  bTerm = new Label (BROAD_LABEL, Label.LEFT);
        Label  nTerm = new Label (NARROW_LABEL, Label.LEFT);
        Label  rTerm = new Label (RELATED_LABEL, Label.LEFT);
             //true means mult select allowed:
        broadTerms = new List (ROWS_VISIBLE, true);
        narrowTerms = new List (ROWS_VISIBLE, true);
        relatedTerms = new List (ROWS_VISIBLE, true);

        p1 = setupList (bTerm, broadTerms);
        p2 = setupList (nTerm, narrowTerms);
        p3 = setupList (rTerm, relatedTerms);

        pan.setLayout ((new GridLayout (1,3,2,0)));
        pan.add (p1);
        pan.add (p2);
        pan.add (p3);

        return  pan;
    }

    Panel createCmdButtons()
    {
        Panel  pan = new Panel();

        bGetTerms = new Button (TERM_BUTTON_LABEL);
        bCopy = new Button (COPY_BUTTON_LABEL);
        bQuit = new Button (QUIT_BUTTON_LABEL);

        pan.setLayout ((new GridLayout (1,3)));
        pan.add (bGetTerms);
        pan.add (bCopy);
        pan.add (bQuit);

        return  pan;
    }

    private void initButtonAction()
    {
        //don't actually need to connect until user chooses a button
        //needing it;  afterwards, we will stay connected, until user
        //is done with window (we may change that later, if windows
        //tend to stay up unused for long periods)
        //
        if (communication == null)
        {
            communication = new ThesServe(this);
        }
        else
            clearMessage();
    }

    public boolean action (Event evt, Object arg)
    {
        if (evt.target instanceof Button)
        {
            String  label = (String)arg;
            if (label.equals (QUIT_BUTTON_LABEL))
            {
                controlWin.closeWebSearchQHelpWin();
                return  true;
            }
            else  if (label.equals (SEARCH_BUTTON_LABEL))
            {
                initButtonAction();
                doSearchCmd();
                return  true;
            }
            else  if (label.equals (TERM_BUTTON_LABEL))
            {
                initButtonAction();
                getTermsCmd();
                return  true;
            }
            else  if (label.equals (COPY_BUTTON_LABEL))
            {
                initButtonAction();
                copyToWebQuery();
                return  true;
            }
            else
                return  false;
        }
        /*
        else  if (evt.target instanceof TextField)
        {
            // don't care what's selected until we need to know for
            // other functions; Ignore.
            return  false;
        }
        else  if (evt.target instanceof List)
        {
            // don't care what's selected until we need to know for
            // other functions; Ignore.
            return  false;
        }
        else  if (evt.target instanceof Checkbox)
        {
            // don't care what's selected until we need to know for
            // other functions; Ignore.
            return  false;
        }
        */
        return  false;
    }

    void displayMessage (String txt)
    {
        msg.setText (txt);
    }

    void clearMessage()
    {
        msg.setText ("");
    }

    /* for ThesaurusClientParent implementation */
    public void displayTServeMsg (String serverMsg)
    {
//TEST  (do we get here?)
        System.out.println (serverMsg);

        msg.setText (serverMsg);
    }

    // display empty text string for data from every thesaurus, and
    // remove all responseIDs.
    void clearOldData()
    {
        for (int i = 0; i < phrSearched.length; i++)
            phrSearched[i].setText ("");
        for (int i = 0; i < bt.length; i++)
            bt[i].setText ("");
        for (int i = 0; i < nt.length; i++)
            nt[i].setText ("");
        for (int i = 0; i < rt.length; i++)
            rt[i].setText ("");
        for (int i = 0; i < responseID.length; i++)
            responseID[i] = "";
    }

    /* For each thesaurus:  returns string actually searched, #broad terms,
       #narrow terms, #related terms, response id, thesaurus id, strength
       of response (ignored).
    */
    void fillDisplayData (int thesID, String[] dataRow)
    {
        try
        {
            phrSearched[thesID].setText (dataRow[0]);
            bt[thesID].setText (dataRow[1]);
            nt[thesID].setText (dataRow[2]);
            rt[thesID].setText (dataRow[3]);
            responseID[thesID] = dataRow[4];
        }
        catch  (ArrayIndexOutOfBoundsException   aoob)
        {
            System.out.println ("Index " + aoob.getMessage() +
                                          " is out of bounds");
            System.out.print ("  (This same exception could be called ");
            System.out.println ("more than once)");

            aoob.printStackTrace();
            if (communication != null)
            {
                communication.disconnect();
                communication = null;
            }
        }
    }

    // only called if all thesauri are searched, and some do not have
    // answers.  If a row is empty, its from no answers, not because it
    // wasn't searched.
    void fillinAllEmptyData (String searchedText)
    {
        for (int i = 0; i < NUM_THESAURI; i++)
        {
            if ((phrSearched[i].getText()).equals (""))
               fillinEmptyData (searchedText, i);
        }
    }

    void fillinEmptyData (String searchedText, int whichThes)
    {
        phrSearched[whichThes].setText (searchedText);
        bt[whichThes].setText ("0");
        nt[whichThes].setText ("0");
        rt[whichThes].setText ("0");
        responseID[whichThes] = "-1";   //not a legal one
    }

    void doSearchCmd()
    {
        clearOldData();

        String  searchFor = getSearchPhrase();
        if (searchFor.equals (""))
        {
            displayMessage ("No phrase to search for");
            return;
        }

        int[]   thesauri = getSearchThes();
/* TEST
        System.out.print ("Thes. to search: ");
        for (int d = 0; d < thesauri.length; d++)
            System.out.print (" " + thesauri[d]);
        System.out.println ("");
*/
        for (int i = 0; i < thesauri.length; i++)
        {
           if (thesauri[i] >= 0)
           {
              String Cmd = searchFor + "\t" + thesauri[i] + "\t" +
                                 RETURN_ASCII;
/* TEST
              System.out.println ("String to search for: " + Cmd);
*/
              ThesCommand c = new ThesCommand (communication.outstream,
                                      ThesCommand.EXPAND, Cmd);
              c.sendCommand();
                  /* If the command is for all thesauri to be searched,
                       multiple answers will be returned; otherwise, if the
                       user wants mult thes. searched, but did not check off
                       all, then multiple queries will occur, with one answer
                       to each one.
                    */
              // System.out.println ("Sent command");

              ThesResponse r = new ThesResponse (communication.instream);
              r.computeResponse();
              if (r.getStatus() == 0)
              {
                  r.makeDataTokens();
/* TEST
                  r.printTokenList();
*/
                  String[][]  data = r.getTokenList();
                 /* for each thes:  returns
                            string actually searched,
                           #broad terms, #narrow terms, #related terms,
                           response id, thesaurus id,
                           strength of response (ignored).
                 */
                  for (int numThes = 0; numThes < data.length; numThes++)
                  {
                      int  thesID = -1;
                      try   {
                          thesID = Integer.parseInt (data[numThes][5]);
                                            //thes ID in 6th position
                      }
                      catch  (NumberFormatException ne)  {
                          System.out.print ("Error converting thesaurus ID ");
                          System.out.println (+ thesID + " to an integer");
                      }
                      if (thesID >= 0)
                      {
                          //r.getNumAnswers() must be > 0 for 'data'
                          //to have dimension
                          //
                          fillDisplayData (thesID, data[numThes]);
                      }
                  }

                  if (data.length == 0)  //num answers == 0
                  {       //fill in zeros in a single row answer
                      if (!(thesauri[0] == ALL_THESAURI))
                      fillinEmptyData (searchFor, thesauri[i]);
                      // if all thes. searched, handle that below
                  }

                  // if multi-row answer (may or may not have zero
                  // answers, but some rows are blank):
                  if ((thesauri[0] == ALL_THESAURI) &&
                                      (data.length < NUM_THESAURI))
                  {
                      fillinAllEmptyData (searchFor);
                  }
                  if (r.getNumAnswers() < 0)
                      System.out.println ("Do Search Error: number of answers is less than zero!");
              }
              else
              {
                  displayMessage ("Error in response to server search command");
              }
           }
           else
              break;  //have reached end of thes. list
        }
    }

    int getCBindex (Checkbox cb)
    {
       int  index = -1;
       String  label = cb.getLabel();
            //this will match one of items in thesName[]

       for (int i = 0; i < NUM_THESAURI; i++)
       {
           if (thesName[i].equals (label))
           {
              index = i;
              break;
           }
       }
       return  index;
    }

    void populateList (List lst, String[] terms)
    {
       if (terms != null)
       {
           for (int i = 0; i < terms.length; i++)
           {
               lst.addItem (terms[i]);
           }
       }
    }

    void getTermsCmd()
    {
       Checkbox  selectedCB = sourceCBgrp.getCurrent();
       if (selectedCB == null)   //none selected
           return;

       int  whichCB = getCBindex (selectedCB);
          // this is the same as the thesaurus ID to the thesaurus server
          // (see "Important Dependency" near the top of this class).

       if (!(responseID[whichCB].equals("-1")))
                         //then valid results exist for this choice
       {
           String  Cmd = whichCB + " " + responseID[whichCB] + " " +
                                              RETURN_ASCII;
           ThesCommand  c = new ThesCommand (communication.outstream,
                                    ThesCommand.GETRESULTS, Cmd);
           c.sendCommand();

           ThesResponse  r = new ThesResponse (communication.instream);
           r.computeResponse();
           if (r.getStatus() == 0)
           {
              r.makeTermsTokens();
/* TEST
              r.printTokenList();
*/
              String[][]  terms = r.getTokenList();

              broadTerms.clear();
              narrowTerms.clear();
              relatedTerms.clear();
              populateList (broadTerms, terms[0]);
              populateList (narrowTerms, terms[1]);
              populateList (relatedTerms, terms[2]);
           }
           else
           {
               displayMessage ("Error in response to server get_terms command");
           }
       }
    }

    void copyToWebQuery()
    {
        String[]   broad = broadTerms.getSelectedItems(),
                   narrow = narrowTerms.getSelectedItems(),
                   related = relatedTerms.getSelectedItems();

        String[]   additionalTerms = new String [
                                broad.length + narrow.length + related.length];

        System.arraycopy (broad, 0, additionalTerms, 0, broad.length);
        System.arraycopy (narrow, 0, additionalTerms, broad.length,
                                         narrow.length);
        System.arraycopy (related, 0, additionalTerms,
                               broad.length + narrow.length, related.length);
/* TEST
        System.out.println ("Terms to copy:");
        for (int t = 0; t < additionalTerms.length; t++)
            System.out.print (" " + additionalTerms[t]);
        System.out.println ("");
*/
        controlWin.writeToSearchArea (additionalTerms);
    }

    // phrase to search the chosen thesauri for
    //
    String getSearchPhrase()
    {
        String  phrase = tfPhrase.getText();
        return  (phrase.trim());
    }

    // index for 'searchThes[]' is same as Server's Thesaurus ID, except
    // ALL_THESAURI code (which also matches server).
    //
    int[] getSearchThes()
    {
        // based on Checkboxes user chose; contains index that server recognizes
        // for the thesauri the user chooses:
        //
        int    searchThes[] = new int[NUM_THESAURI];

        int j = 0;
        for (int i = 0; i < NUM_THESAURI; i++)
        {
            if (thes[i].getState() == true)
            {
                searchThes[j] = i;
                j++;
            }
        }
        // make rest of array hold '-1', meaning "no value".
        if (j == NUM_THESAURI)   //if all thesauri searched:
        {
            searchThes[0] = ALL_THESAURI;
            for (j = 1; j < NUM_THESAURI; j++)
                searchThes[j] = -1;
        }
        else
            for ( ; j < NUM_THESAURI; j++)
                searchThes[j] = -1;
/* TEST
        System.out.println ("Search Thesauri List:");
        System.out.print ("  ");
        for (int k = 0; k < NUM_THESAURI; k++)
            System.out.print (searchThes[k] + " ");
        System.out.println ("");
*/
        return   searchThes;
    }

    int getExpandThes()
    {
        int    expandThes = -1;

        for (int i = 0; i < NUM_THESAURI; i++)
        {
            if (src[i].getState() == true)
            {
                expandThes = i;
                break;
            }
        }
        return  expandThes;
    }
}
