/* Bordered Panel class
   allows us to change insets as desired for components laid out in this class.
*/
package  UserInt;

import   java.awt.*;

/** A panel which can have a black border. */
public class BorderedPanel extends Panel
{
    private Insets  insetData;
    private boolean hasBorder;
    private boolean visibleChild;  //if has a border, only draw if this is true
        //assumes, for now, once visible, always visible.

    public BorderedPanel (int top, int left, int bottom, int right)
    {
        // top, left, bottom, right
        insetData = new Insets(top, left, bottom, right);
        hasBorder = false;
        visibleChild = false;
    }

    //override insets() in Container
    // we should change this to getInsets() for Java 1.1
    public Insets insets()
    {
        return  (insetData);
    }

    public void setHasBorder()
    {
        hasBorder = true;
    }

    public void clearHasBorder()
    {
        hasBorder = false;
    }

    public void setVisibleComponents()
    {
        visibleChild = true;
    }

    public void paint (Graphics g)
    {
      /* why do we only call the super's paint() for Java 1.1? */
        if (ApplType.getIsVersion1_1())
            super.paint (g);
               //TEST  System.out.println ("Paint called");

        if (hasBorder && visibleChild)
        {
            Dimension  d = size();
            Insets    in = insets();

            if (g != null)
            {
                boolean resetColor = false;
                Color c = g.getColor();
                if (c != Color.black)
                {
	                 resetColor = true;
                    g.setColor(Color.black);
                    //TEST  g.setColor(Color.red);
                }
                int  rightX = d.width - in.right;
                int  bottomY = d.height - in.bottom;
                g.drawLine (0, 0, rightX, 0);
                g.drawLine (0, 0, 0, bottomY);
                g.drawLine (0, bottomY, rightX, bottomY);
                g.drawLine (rightX, 0, rightX, bottomY);
                if (resetColor)
                    g.setColor(c);
            }
	}
    }

    public void reshape (int x, int y, int width, int height)
    {
        super.reshape(x, y, width, height);
        if (hasBorder)
        {
            //TEST  System.out.println ("Reshape calls 'Repaint'");

            if (getGraphics() != null)
                repaint();
        }
   }
}
