/* Edit Window
 * Copyright (C) 1999 - Jeff Heflin - University of Maryland
 *
 * This source code may be redistributed or modified provided that
 * this notice is unchanged and all modifications are clearly indicated.
 * No portion of this code may be included in a commercial product without
 * the written permission of the author.
 *
 * The software and related materials are provided "as-is" without
 * warranty or indemnity of any kind, including any warranties of
 * use, performance or merchantability or fitness for a particular use
 * or purpose or for any purpose whatsoever. User bears all risk
 * relating to use, quality and performance of the software.
 *
 * Author: Jeff Heflin
 * e-mail: heflin@cs.umd.edu
 *
 */

package UserInt;

import java.awt.*;
import java.lang.System;

/** This class is used to create pop-up windows that allow a user to change
  or set some parameters */
public abstract class EditWindow extends WorkaroundDialog {
// public abstract class EditWindow extends Dialog {

  Button b_OK, b_Cancel;

  public EditWindow(Frame parent, String title) {

    super(parent, title, true);
    // super(parent, title);

    Panel butPanel = new Panel();
    b_OK = new Button("OK");
    b_Cancel = new Button("Cancel");
    butPanel.add(b_OK);
    butPanel.add(b_Cancel);
    add("South", butPanel);
    setBackground(Color.lightGray);
    setForeground(Color.black);
    pack();
    resize(minimumSize());
  }

  public boolean action(Event e, Object arg) {

    String fieldErrs;

    Object target = e.target;

    if (target instanceof Button) {
      if (target == b_OK) {
	fieldErrs = fieldErrors();
	if (fieldErrs == null) {
	  saveChanges();
	  dispose();
	}
	else {
	  Notification note;
	  // String buttons[] = {"OK"};
	  // String text[] = {fieldErrs};
	  note = new Notification((Frame)getParent(), "Error", fieldErrs);
	  note.showWorkaround();
	}	    
        return true;
      }
      else if (target == b_Cancel) {
        dispose();
        return true;
      }
    }
    return false;
  }
  
  public boolean handleEvent(Event event) {

    if (event.id == Event.WINDOW_DESTROY) {
      dispose();
    }
    return super.handleEvent(event);
  }

  /** Sub-classes must provide code to save the contents of the window */
  public abstract void saveChanges();

  /** Returns a string describing any errors in the fields that were
    edited, if no errors then returns null. Subclasses can override
    this function to provide their own  validation mechanisms */
  public String fieldErrors() {

    return null;
  }
}  


