package UserInt;

import java.awt.*;
import java.util.Vector;
import java.lang.System;

public class ObjectList extends List {
  /* A list that has an Object associated with each entry. The list allows
     the selected Object to be retrieved as well as the index or the label */

  protected Vector oblist;
  
  public ObjectList(int rows, boolean multiple) {
    super(rows, multiple);
    setBackground(Color.white);
    oblist = new Vector();
  }

  public void addItem(String label, Object obj) {

    super.addItem(label);
    // insert object at index one less than the number of labels
    oblist.insertElementAt(obj, countItems() - 1);
  }

  public void addItem(String label, Object obj, int index) {

    // System.out.println("ObjectList.addItem(\"" + label + "\", Object, " + index + ") called");
    super.addItem(label, index);
    oblist.insertElementAt(obj, index);
  }

  public void addItem(String label) {

    // System.out.println("ObjectList.addItem(\"" + label + "\") called");
    super.addItem(label);
  }

  public void addItem(String label, int index) {
    /* addItem(String, int) is called by addItem(String) */

    // System.out.println("ObjectList.addItem(\"" + label + "\", " + index +  ") called");
    super.addItem(label, index);
  }

  public Object getSelectedObject() {

    int index;

    index = getSelectedIndex();
    if (index >= 0)
      return oblist.elementAt(index);
    else
      return null;
  }

  public Object getObject(int index) {

    return oblist.elementAt(index);
  }

  public Vector getObjects() {

    return oblist;
  }

  /* Replaces the item by removing all elements after it in the list,
     and adding them back. When this happens, the list repositions to the
     first element. We would like to use getVisibleIndex() to find the
     visible item before hand, and then use makeVisible() to return the
     list to its original position, but the functions don't work that way.
     Instead, we settle for making sure that the selected item is visible.
     It will appear at the bottom of the visible list if not in the first
     few items */
  public void replaceItem(String newStr, Object newObj, int index) {

    int i;
    int vis_index = -1;

    // System.out.println("ObjectList.replaceItem(\"" + newStr + "\", Object, " + index + ") called");
    Vector oldlist = (Vector)oblist.clone();

    /* remember what part of list was visible. For some dumb reason, this
       might only remember indices you set with makeVisible() */
    /* vis_index = getVisibleIndex();           
    // System.out.println("vis_index = " + vis_index);
    */

    // No getItems() in Java 1.0. This calls the function below instead
    String oldlabels[] = getItems();

    delItems(index, countItems() - 1);
    addItem(newStr, newObj);
    for (i = index + 1; i < oldlist.size(); i++)
      addItem(oldlabels[i], oldlist.elementAt(i));
    select(index);
    if (vis_index >= 0)
      makeVisible(vis_index);   // return list to original position
    else
      makeVisible(index);  // or at least make the selection visible
  }
    
  /* getItems() is not available til Java 1.1. Therefore, we provide
     a version of it here for Java 1.0 */
  public String[] getItems() {
    
    int i;
    String[] itemlist;
    int size;
    
    size = countItems();
    itemlist = new String[size];
    for (i=0; i < size; i++)
      itemlist[i] = getItem(i);
    return itemlist;
  }

  public void delItem(int index) {

    // System.out.println("ObjectList.delItem(" + index + ") called");
    delItems(index, index);
  }

  public void delItems(int low, int high) {
    /* delItems is called by delItem. */
    int i;

    // System.out.println("ObjectList.delItems(" + low + ", " + high + ") called");
    super.delItems(low, high);
    // remove objects in reverse order so indices don't change
    for (i=high; i >= low; i--)     
      oblist.removeElementAt(i);
  }
  
  public void clear() {

    int numItems = countItems();

    if (numItems > 0)
      delItems(0, numItems - 1);
  }

}
